// datalist.dart
import 'dart:convert';
import 'package:flutter/widgets.dart';
import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:pull_to_refresh/pull_to_refresh.dart';

class DataList extends StatefulWidget {
  final String search;
  final String endpoint;
  final int defaultLimit;
  final String limitParam;
  final Function child;
  final Function convert; // convert json to dynamic data

  DataList({
    Key key,
    this.child,
    this.search,
    this.convert,
    this.endpoint,
    this.defaultLimit: 10,
    this.limitParam,
  }) : super(key: key);

  @override
  _DataListState createState() => _DataListState();
}

class _DataListState extends State<DataList> {
  Future<List<dynamic>> _futureData;
  RefreshController _refreshController =
      RefreshController(initialRefresh: false);
  int limit = 0;

  @override
  void initState() {
    super.initState();
    print('init ${widget.search}');
    if(limit == 0) limit = widget.defaultLimit;
    _futureData = fetchPost(limit, widget.search);
  }

  void _onRefresh() async {
    _futureData = fetchPost(20, '');
    await Future.delayed(Duration(milliseconds: 1000));
    _refreshController.refreshCompleted();
  }

  void _onLoading() async {
    _futureData = fetchPost(limit + widget.defaultLimit, widget.search);
    await Future.delayed(Duration(milliseconds: 1000));
    if (mounted)
      setState(() {
        limit = limit + widget.defaultLimit;
      });
    _refreshController.loadComplete();
  }

  Future<List<dynamic>> fetchPost(limit, search) async {
    String limitStr = '?limit=' + limit.toString();
    final response = await http.get('${widget.endpoint}$limitStr$search');
    if (response.statusCode == 200) {
      List jsonResponse = json.decode(response.body);
      return jsonResponse.map((res) => widget.convert(res)).toList();
    } else {
      throw Exception('Failed to load jobs from API');
    }
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<List<dynamic>>(
      future: _futureData,
      builder: (context, snapshot) {
        if (snapshot.hasData) {
          List<dynamic> data = snapshot.data;
          return _listView(data);
        } else if (snapshot.hasError) {
          return Text("${snapshot.error}");
        }
        return CircularProgressIndicator();
      },
    );
  }

  Widget _listView(data) {
    return SmartRefresher(
      enablePullUp: true,
      enablePullDown: true,
      onRefresh: _onRefresh,
      onLoading: _onLoading,
      child: widget.child(data),
      controller: _refreshController,
      header: WaterDropMaterialHeader(),
    );
  }
}
